@@ -13,7 +13,7 @@ import sqlalchemy as sql
 from aiojobs.aiohttp import spawn
 from aiohttp import web
 from cryptography.fernet import InvalidToken
-from marshmallow import fields, validate
+from marshmallow import fields, validate, validates, ValidationError
 from psycopg2 import errors as postgres_errors
 from sqlalchemy.dialects.postgresql import JSONB
 
@@ -90,6 +90,7 @@ async def send_email_user_invitation(request, by, link, to):
         username=os.environ.get('EMAIL_USERNAME'),
         password=os.environ.get('EMAIL_PASSWORD')
     )
+    request.app.info_logger.info("Invitation email sent", invited=to)
 
 
 async def send_email_reset_link(request, checkcode, to):
@@ -134,6 +135,7 @@ async def send_email_reset_link(request, checkcode, to):
         username=os.environ.get('EMAIL_USERNAME'),
         password=os.environ.get('EMAIL_PASSWORD')
     )
+    request.app.info_logger.info("Sent password reset link", email=to)
 
 
 async def send_email_verification_link(request, to):
@@ -175,6 +177,7 @@ async def send_email_verification_link(request, to):
         username=os.environ.get('EMAIL_USERNAME'),
         password=os.environ.get('EMAIL_PASSWORD')
     )
+    request.app.info_logger.info("Sent email verification link", email=to)
 
 
 async def send_email_activation_link(request, data, link_path_base, ttl_seconds):
@@ -219,6 +222,7 @@ async def send_email_activation_link(request, data, link_path_base, ttl_seconds)
         username=os.environ.get('EMAIL_USERNAME'),
         password=os.environ.get('EMAIL_PASSWORD')
     )
+    request.app.info_logger.info("Activation email sent", email=data['email'], username=data['username'])
 
 
 async def send_phone_verification_code(request, phone_num, actor_id):
@@ -764,14 +768,19 @@ class InviteUser(AuxView):
 class GrantRole(AuxView):
     actor_id = fields.Int(location='path')  # grantee
     roles = postschema_field.Set(
-        fields.String(
-            validate=[validate.OneOf(ROLES)]
-        ),
+        fields.String(),
         location='body',
         sqlfield=JSONB,
         required=True
     )
 
+    @validates('roles')
+    def roles_validator(self, val):
+        delta = set(val) - self.app.allowed_roles
+        if delta:
+            bad_roles = ', '.join(delta)
+            raise ValidationError(f'Invalid roles: {bad_roles}')
+
     @summary('Update user\'s roles')
     async def patch(self):
         actor_id = self.path_payload['actor_id']
@@ -1171,9 +1180,7 @@ class PrincipalActorBase(RootSchema):
     email_confirmed = fields.Boolean(sqlfield=sql.Boolean, read_only=True)
     password = fields.String(sqlfield=sql.String(555), required=True, validate=validate.Length(min=6))
     roles = fields.List(
-        fields.String(
-            validate=[validate.OneOf(ROLES)]
-        ),
+        fields.String(),
         validate=[validators.must_not_be_empty],
         sqlfield=JSONB
     )
@@ -1181,10 +1188,18 @@ class PrincipalActorBase(RootSchema):
     otp_secret = fields.String(sqlfield=sql.String(24))
     details = fields.Dict(sqlfield=JSONB)
 
+    @validates('roles')
+    def roles_validator(self, val):
+        delta = set(val) - self.app.allowed_roles
+        if delta:
+            bad_roles = ', '.join(delta)
+            raise ValidationError(f'Invalid roles: {bad_roles}')
+
     async def before_update(self, parent, request, payload, selector):
         if 'phone' in payload:
             payload['phone'] = clean_phone_number(payload['phone'])
         if 'details' in payload:
+            payload['details'] = payload['details'].adapted
             scope = request.session.scope
             scope_inst = request.app.config.scopes[scope]
             details = await parent._validate_singular_payload(
@@ -1389,6 +1404,8 @@ class PrincipalActorBase(RootSchema):
         return request.app.created_email_confirmation_link, request.app.config.activation_link_ttl
 
     async def before_post(self, parent, request, data):
+        if 'details' in data:
+            data['details'] = data['details'].adapted
         data['otp_secret'] = pyotp.random_base32(24)
         query = request.query
         invitation_token = query.get('inv')
@@ -1410,6 +1427,9 @@ class PrincipalActorBase(RootSchema):
         await spawn(request, send_email_activation_link(request, data, link_path_base, ttl))
         raise web.HTTPNoContent(reason='Account creation pending')
 
+    class AccessLogging:
+        authed = '*'
+
     class Public:
         disallow_authed = ['post']
 
