@@ -128,6 +128,47 @@ def make_select_fields_schema(schema_cls):
 
 
 class CommonViewMixin:
+
+    @classmethod
+    async def log_request(cls, req, resp):
+        logging_cls = getattr(cls.schema_cls, 'AccessLogging', None)
+        if not logging_cls:
+            return
+
+        is_authed = req.session.needs_session
+        if not is_authed:
+            pub_logging_conf = getattr(logging_cls, 'public', None)
+            if not pub_logging_conf or req.operation not in pub_logging_conf:
+                return
+        else:
+            auth_logging_conf = getattr(logging_cls, 'authed', None)
+            if not auth_logging_conf or req.operation not in auth_logging_conf:
+                return
+
+        status = resp.status
+        if status >= 400:
+            method = 'error'
+            txt_resp = str(resp.reason)
+        else:
+            txt_resp = str(resp.text)
+            method = 'info'
+
+        txt_resp = str(resp.reason)
+        try:
+            payload = await req.json()
+        except Exception:
+            payload = {}
+        msg = dict(
+            resp=txt_resp[:5000] + '...' if resp.body_length >= 5000 else txt_resp,
+            payload=payload,
+            **req.app.access_logger._context.pop('msg_context', {})
+        )
+        await getattr(req.app.access_logger, method)(
+            msg,
+            status=status,
+            path=req.path,
+            op=req.operation)
+
     async def _validate_singular_payload(self, payload=None, schema=None, envelope_key=None,
                                          raise_orig=False):
         ref_schema = schema if schema is not None else self.schema
@@ -157,7 +198,7 @@ class CommonViewMixin:
             raise post_exceptions.ValidationError(err_msg if not envelope_key else {envelope_key: err_msg})
 
         with suppress(AttributeError):
-            # ignore validating \w Schemas not inheriting from PostSchema
+            # ignore validating \w schemas not inheriting from PostSchema
             err_msg = await ref_schema.run_async_validators(payload_used) or err_msg
 
         if err_msg:
@@ -350,8 +391,8 @@ class ViewsClassBase(web.View):
         schema = cls.schema_cls
         new_methods = {
             fieldname: fields.Nested(
-                linked_schema, validate=must_not_be_empty)
-            for fieldname, (linked_schema, *_) in joins.items()
+                join_obj['linked_schema'], validate=must_not_be_empty)
+            for fieldname, join_obj in joins.items()
         }
 
         for fieldname in joins:
@@ -556,11 +597,13 @@ class ViewsClassBase(web.View):
         tablename = schema.__tablename__
         get_joins = {}
         list_joins = {}
-        for fieldname, (linked_schema, _, target_table) in joins.items():
+        for fieldname, join_obj in joins.items():
+            linked_schema = join_obj['linked_schema']
+            target_table = join_obj['target_table']
             linked_schema_tablename = linked_schema.__tablename__
             target_col = target_table['target_col']
-            linked = f'"{linked_schema_tablename}".{target_col}'
-            join_stmt = f'LEFT JOIN "{linked_schema_tablename}" ON "{tablename}".{fieldname}={linked}'
+            linked = f'_{fieldname}_j.{target_col}'
+            join_stmt = f'LEFT JOIN "{linked_schema_tablename}" _{fieldname}_j ON "{tablename}".{fieldname}={linked}'
             if fieldname in get_by:
                 get_joins[fieldname] = join_stmt
             if fieldname in list_by:
@@ -585,18 +628,14 @@ class ViewsClassBase(web.View):
 
         metacls_name = request_type.title()
 
-        field_to_table = {
-            field: li[0]._model.__table__.name
-            for field, li in cls.schema_cls._join_to_schema_where_stmt.items()
-        }
-
         def _join_selects(select_dict, tablename):
             return ','.join(
                 f"'{k}', \"{tablename}\".{v}"
                 if not isinstance(v, dict)
-                else f"'{k}', json_build_object({_join_selects(v, field_to_table[k])})"
+                else f"'{k}', json_build_object({_join_selects(v, '_' + k + '_j')})"
                 for k, v in select_dict.items()
             )
+
         tablename = cls.schema_cls.__tablename__
         tablename_cte = f'{tablename}_cte'
         joined_fields = dd(dict)
@@ -607,7 +646,7 @@ class ViewsClassBase(web.View):
 
         for getter_field in list_by.copy():
             if getter_field in joins_to_schemas:
-                linked_schema = joins_to_schemas[getter_field][0]
+                linked_schema = joins_to_schemas[getter_field]['linked_schema']
                 popped_field = list_by.pop(getter_field, None)
                 if not popped_field:
                     continue
@@ -662,16 +701,11 @@ class ViewsClassBase(web.View):
 
         metacls_name = request_type.title()
 
-        field_to_table = {
-            field: li[0]._model.__table__.name
-            for field, li in cls.schema_cls._join_to_schema_where_stmt.items()
-        }
-
         def _join_selects(select_dict, tablename):
             return ','.join(
                 f"'{k}', \"{tablename}\".{v}"
                 if not isinstance(v, dict)
-                else f"'{k}', json_build_object({_join_selects(v, field_to_table[k])})"
+                else f"'{k}', json_build_object({_join_selects(v, '_' + k + '_j')})"
                 for k, v in select_dict.items()
             )
 
@@ -682,7 +716,7 @@ class ViewsClassBase(web.View):
 
         for getter_field in get_by.copy():
             if getter_field in joins_to_schemas:
-                linked_schema = joins_to_schemas[getter_field][0]
+                linked_schema = joins_to_schemas[getter_field]['linked_schema']
                 popped_field = get_by.pop(getter_field, None)
                 if not popped_field:
                     continue
@@ -1008,7 +1042,7 @@ class ViewsBase(ViewsClassBase, CommonViewMixin):
         tablename = self.schema.__tablename__
         joins = []
         usings = []
-        froms = [] # for updates only
+        froms = []  # for updates only
         wheres = deque()
         values = {}
 
@@ -1027,13 +1061,15 @@ class ViewsBase(ViewsClassBase, CommonViewMixin):
             if relation_in_payload:
                 values.update({m2m_field: relation_in_payload})
                 wheres.append(m2m_field_translated)
-
-        for fk_field, (linked_schema, where_stmt, _) in self.schema._join_to_schema_where_stmt.items():
+        
+        for fk_field, join_obj in self.schema._join_to_schema_where_stmt.items():
+            linked_schema = join_obj['linked_schema']
             if fk_field in self.tables_to_join:
                 joins.append(self.schema._joins[fk_field])
                 usings.append(fk_field)
             fk_in_payload = cleaned_payload.pop(fk_field, None)
             if fk_in_payload:
+                where_stmt = join_obj['unaliased_comp_query'] if in_update or in_delete else join_obj['aliased_comp_query']
                 with suppress(AttributeError):
                     # if <schema>.Meta defines a `default_get_critera` function
                     # which in turn returns an expected FK value, we can ignore this
@@ -1062,6 +1098,7 @@ class ViewsBase(ViewsClassBase, CommonViewMixin):
             using = f'USING {using}'
         if froms:
             froms = f'FROM "{froms}"'
+
         return query.format(where=wheres_q, joins=joins, using=using, froms=froms), values
 
 
@@ -1173,10 +1210,11 @@ class ViewsTemplate:
                 if res is None:
                     if self.request.session:
                         # Most likely a cross workspace insert or non-existent FK
-                        self.request.app.error_logger.warn('Cross workspace insert')
                         raise web.HTTPConflict(reason='Illegal cross workspace insert or non-existent FK supplied')
-                    self.request.app.error_logger.error("Failed to create a resource",
-                                                        query=cur.query.decode())
+
+                    self.request.app.access_logger = self.request.app.access_logger.bind(msg_context={
+                        'query': cur.query.decode()
+                    })
                     raise post_exceptions.CreateFailed()
 
         if hasattr(self.schema, 'after_post'):
@@ -1207,8 +1245,9 @@ class ViewsTemplate:
                 await self.request.app.commons.execute(cur, query, query_values, envelope='payload')
                 res = await cur.fetchone()
                 if not res or not res[0]:
-                    self.request.app.error_logger.error("Failed to update a resource",
-                                                        query=cur.query.decode())
+                    self.request.app.access_logger = self.request.app.access_logger.bind(msg_context={
+                        'query': cur.query.decode()
+                    })
                     raise post_exceptions.UpdateFailed()
 
         if hasattr(self.schema, 'after_put'):
@@ -1246,8 +1285,9 @@ class ViewsTemplate:
                 await self.request.app.commons.execute(cur, query, query_values, envelope='payload')
                 res = await cur.fetchone()
                 if not res or not res[0]:
-                    self.request.app.error_logger.error('Failed to update a resource',
-                                                        query=cur.query.decode())
+                    self.request.app.access_logger = self.request.app.access_logger.bind(msg_context={
+                        'query': cur.query.decode()
+                    })
                     raise post_exceptions.UpdateFailed()
 
         if hasattr(self.schema, 'after_patch'):
@@ -1303,8 +1343,9 @@ class ViewsTemplate:
                     # fetch the query result under the same transaction, before commiting
                     res = await cur.fetchone()
                     if not res or not res[0]:
-                        self.request.app.error_logger.error('Failed to delete a resource',
-                                                            query=cur.query.decode())
+                        self.request.app.access_logger = self.request.app.access_logger.bind(msg_context={
+                            'query': cur.query.decode()
+                        })
                         raise post_exceptions.DeleteFailed()
                     deleted_ids = res[0]
 
